#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include "si_prefix.h"


/** sign(x) */
#define sign(x) (((x)<0)?-1:((x)>0?1:0))


/** SI-Einheiten */
const char * SI_PREFIX[] = { "a", "f", "p", "n", "µ", "m", "", 
                             "k", "M", "G", "T", "P", "E" };

/**
 * determine Si prefix
 *
 * x	    number
 * d	    number of significant digits
 * return: prefix and factor
 */
t_si_prefix si_prefix( double x, int d )
{
    t_si_prefix p;
    double l, vz;
    int e;

    /* init */
    p.pref[0] = 0;
    p.factor = 1;
    vz = sign( x );
    /* test for zero */
    if ( vz == 0 ) 
    {
	p.prec = d-1;
	return p;
    }
    /* determine prefix */
    l = log( vz * x ) / log( 1000 );
    e = (int)floor( l );
    if ( e > 6 || e < -6 ) 
    {
	p.prec = d;
	return p;
    }
    strcpy( p.pref, SI_PREFIX[e+6] );
    p.factor = pow( 1000, e );
    /* determine precision */
    p.prec = d - 1 -(int)floor( log10( vz*x/p.factor ) );
    return p;
}

/**
 * format quantity: number and unit
 *
 * The return string varies in length. Due to utf8-chars the lenght does not
 * correspond to positions on screen.
 * '-0.00 µOhm'
 * '-00.0 MS'
 *
 * x	    number
 * p	    SI prefix
 * u	    unit, at most three chars
 * return: string in static buffer
 *         The buffer is overwritten at the next call.
 */
char * format_si( double x, t_si_prefix p, const char * u )
{
    static char buf[20];
   
    x = x / p.factor;
    snprintf( buf, 20, "% 5.*f %s%s", p.prec, x, p.pref, u );
    return buf;
}

/**
 * read number and SI-prefix
 *
 * Read a string like "100 k" as 1e5.
 * 
 * s	    string
 * return: number
 */
double scan_si( const char * s )
{
    double x, f;
    char * prefix;
    int i;

    /* get number */
    x = strtod( s, &prefix );
    /* discard whitespace after number, before prefix */
    while ( prefix[0] != 0 && isspace( prefix[0] ) )
	prefix++;
    /* get factor from prefix */
    f = 1;
    for ( i = 0; i < 13; i++ )
	if ( SI_PREFIX[i][0] != 0 && strstr( prefix, SI_PREFIX[i] ) )
	{
	    f = pow( 1000, i-6 );
	    break;
	}
    return x * f;
}


